----- MODULE Cons -----
EXTENDS Naturals, TLC, Sequences
CONSTANT N

NONE == CHOOSE n : n \notin {0, 1}

(* --algorithm ConsAlg
	variables pvalues = [i \in 1..N |-> NONE], values = [i \in 1..N |-> NONE], R = NONE;

	macro WRITE(val)
	begin
		if R = NONE then R := val end if;
	end macro

	procedure propose(pval)
	begin
		write:	WRITE(pval);
		decide:	values[self] := R;
		ret:	return;
	end procedure
	
	process Proc \in 1..N
	begin
		pval:	either pvalues[self] := 0 or pvalues[self] := 1 end either;
		prop:	call propose(pvalues[self]);
	end process
	
	end algorithm
*)

\* BEGIN TRANSLATION
\* Label pval of process Proc at line 24 col 25 changed to pval_
VARIABLES pvalues, values, R, pc, stack, pval

vars == << pvalues, values, R, pc, stack, pval >>

ProcSet == (1..N)

Init == (* Global variables *)
        /\ pvalues = [i \in 1..N |-> NONE]
        /\ values = [i \in 1..N |-> NONE]
        /\ R = NONE
        (* Procedure propose *)
        /\ pval = [ self \in ProcSet |-> {}]
        /\ stack = [self \in ProcSet |-> << >>]
        /\ pc = [self \in ProcSet |-> CASE self \in 1..N -> "pval_"]

write(self) == /\ pc[self] = "write"
               /\ IF R = NONE
                     THEN /\ R' = pval[self]
                     ELSE /\ TRUE
                          /\ UNCHANGED R
               /\ pc' = [pc EXCEPT ![self] = "decide"]
               /\ UNCHANGED << pvalues, values, stack, pval >>

decide(self) == /\ pc[self] = "decide"
                /\ values' = [values EXCEPT ![self] = R]
                /\ pc' = [pc EXCEPT ![self] = "ret"]
                /\ UNCHANGED << pvalues, R, stack, pval >>

ret(self) == /\ pc[self] = "ret"
             /\ pc' = [pc EXCEPT ![self] = Head(stack[self]).pc]
             /\ pval' = [pval EXCEPT ![self] = Head(stack[self]).pval]
             /\ stack' = [stack EXCEPT ![self] = Tail(stack[self])]
             /\ UNCHANGED << pvalues, values, R >>

propose(self) == write(self) \/ decide(self) \/ ret(self)

pval_(self) == /\ pc[self] = "pval_"
               /\ \/ /\ pvalues' = [pvalues EXCEPT ![self] = 0]
                  \/ /\ pvalues' = [pvalues EXCEPT ![self] = 1]
               /\ pc' = [pc EXCEPT ![self] = "prop"]
               /\ UNCHANGED << values, R, stack, pval >>

prop(self) == /\ pc[self] = "prop"
              /\ /\ pval' = [pval EXCEPT ![self] = pvalues[self]]
                 /\ stack' = [stack EXCEPT ![self] = << [ procedure |->  "propose",
                                                          pc        |->  "Done",
                                                          pval      |->  pval[self] ] >>
                                                      \o stack[self]]
              /\ pc' = [pc EXCEPT ![self] = "write"]
              /\ UNCHANGED << pvalues, values, R >>

Proc(self) == pval_(self) \/ prop(self)

Next == (\E self \in ProcSet: propose(self))
           \/ (\E self \in 1..N: Proc(self))
           \/ (* Disjunct to prevent deadlock on termination *)
              (\A self \in ProcSet: pc[self] = "Done" /\ UNCHANGED vars)

Spec == Init /\ [][Next]_vars

Termination == <>(\A self \in ProcSet: pc[self] = "Done")

\* END TRANSLATION

Agreement == \A i, j \in 1..N : (values[i] = NONE) \/ (values[j] = NONE) \/ (values[i] = values[j])

Validity == \A i \in 1..N : (values[i] # NONE) => ( \E j \in 1..N : (values[i] = pvalues[j]) )

=====
